@@ -90,15 +90,13 @@ class SQLiteDatabaseNewForm(Database):
     def listtags(self, alltags=False):
         sql = ''
         params = []
-        if len(self._filtertags) == 0 or alltags:
+        if not self._filtertags or alltags:
             sql = "SELECT DATA FROM TAGS ORDER BY DATA ASC"
         else:
             sql = ("SELECT TAGS.DATA FROM LOOKUP"
                    + " INNER JOIN TAGS ON LOOKUP.TAG = TAGS.ID"
                    + " WHERE NODE IN (")
             first = True
-            # if using the command filter, the code crashes ...
-            #
             for t in self._filtertags:
                 if not first:
                     sql += " INTERSECT "
@@ -107,7 +105,7 @@ class SQLiteDatabaseNewForm(Database):
 
                 sql += ("SELECT NODE FROM LOOKUP LEFT JOIN TAGS ON TAG = "
                         + " TAGS.ID WHERE TAGS.DATA = ?")
-                params.append(t.get_name())
+                params.append(t._name)
             sql += ") EXCEPT SELECT DATA FROM TAGS WHERE "
             first = True
             for t in self._filtertags:
@@ -116,7 +114,7 @@ class SQLiteDatabaseNewForm(Database):
                 else:
                     first = False
                 sql += "TAGS.DATA = ?"
-                params.append(t)
+                params.append(t._name)
         try:
             self._cur.execute(sql, params)
             tags = []
@@ -128,6 +126,9 @@ class SQLiteDatabaseNewForm(Database):
             return tags
         except sqlite.DatabaseError, e:
             raise DatabaseException("SQLite: %s" % (e))
+        except sqlite.InterfaceError, e:
+            import ipdb
+            ipdb.set_trace()  # XXX BREAKPOINT
 
     def parse_node_string(self, string):
         nodestring = string.split("##")
@@ -154,8 +155,8 @@ class SQLiteDatabaseNewForm(Database):
                     nodestring = str(row[0])
                     nodeargs, tags = self.parse_node_string(nodestring)
                     node = NewNode(**nodeargs)
-                    node.set_tags(tags)
-                    node.set_id(i)
+                    node.tags = tags
+                    node._id = i
                     nodes.append(node)
         return nodes
 
@@ -188,7 +189,7 @@ class SQLiteDatabaseNewForm(Database):
             except sqlite.DatabaseError, e:
                 raise DatabaseException("SQLite: %s" % (e))
             idx = self._cur.lastrowid
-            n.set_id(idx)
+            n._id = idx
             self._setnodetags(n)
             self._commit()
 
@@ -198,7 +199,7 @@ class SQLiteDatabaseNewForm(Database):
             #    "Tried to delete foreign object from database [%s]", n)
             try:
                 sql = "DELETE FROM NODES WHERE ID = ?"
-                self._cur.execute(sql, [n.get_id()])
+                self._cur.execute(sql, [n._id])
 
             except sqlite.DatabaseError, e:
                 raise DatabaseException("SQLite: %s" % (e))
@@ -221,7 +222,6 @@ class SQLiteDatabaseNewForm(Database):
                     first = False
                 sql += ("SELECT NODE FROM LOOKUP LEFT JOIN TAGS ON TAG = "
                         " TAGS.ID WHERE TAGS.DATA = ? ")
-                #params.append(t.get_name())
                 params.append(t._name)
         try:
             self._cur.execute(sql, params)
@@ -242,10 +242,26 @@ class SQLiteDatabaseNewForm(Database):
             raise DatabaseException(
                 "SQLite: Error commiting data to db [%s]" % (e))
 
+    def _create_tag(self, tag, node_ids):
+        """add tags to db"""
+        sql = "INSERT OR REPLACE INTO TAGS(DATA) VALUES(?)"
+        if isinstance(tag, str):
+            self._cur.execute(sql, [tag])
+        else:
+            self._cur.execute(sql, [tag._name])
+
+    def _update_tag_lookup(self, node, tag_id):
+        sql = "INSERT OR REPLACE INTO LOOKUP VALUES(?, ?)"
+        params = [node._id, tag_id]
+        try:
+            self._cur.execute(sql, params)
+        except sqlite.DatabaseError, e:
+            raise DatabaseException("SQLite: %s" % (e))
+
     def _tagids(self, tags):
         ids = []
+        sql = "SELECT ID FROM TAGS WHERE DATA = ?"
         for tag in tags:
-            sql = "SELECT ID FROM TAGS WHERE DATA = ?"
             try:
                 if isinstance(tag, str):
                     self._cur.execute(sql, [tag])
@@ -255,12 +271,7 @@ class SQLiteDatabaseNewForm(Database):
                 if (row is not None):
                     ids.append(row[0])
                 else:
-                    sql = "INSERT INTO TAGS(DATA) VALUES(?)"
-                    if isinstance(tag, str):
-                        self._cur.execute(sql, [tag])
-                    else:
-                        self._cur.execute(sql, [tag._name])
-
+                    self._create_tag(tag, None)
                     ids.append(self._cur.lastrowid)
             except sqlite.DatabaseError, e:
                 raise DatabaseException("SQLite: %s" % (e))
@@ -269,22 +280,16 @@ class SQLiteDatabaseNewForm(Database):
     def _deletenodetags(self, node):
         try:
             sql = "DELETE FROM LOOKUP WHERE NODE = ?"
-            self._cur.execute(sql, [node.get_id()])
+            self._cur.execute(sql, [node._id])
         except sqlite.DatabaseError, e:
             raise DatabaseException("SQLite: %s" % (e))
         self._commit()
 
     def _setnodetags(self, node):
         self._deletenodetags(node)
-        ids = self._tagids(node.get_tags())
-        for i in ids:
-            sql = "INSERT OR REPLACE INTO LOOKUP VALUES(?, ?)"
-            params = [node.get_id(), i]
-
-            try:
-                self._cur.execute(sql, params)
-            except sqlite.DatabaseError, e:
-                raise DatabaseException("SQLite: %s" % (e))
+        ids = self._tagids(node.tags)
+        for tagid in ids:
+            self._update_tag_lookup(node, tagid)
         self._commit()
 
     def _checktags(self):
