@@ -29,6 +29,7 @@ import pwman.util.generator as generator
 from pwman.data.nodes import Node
 from pwman.data.nodes import NewNode
 from pwman.data.tags import Tag
+from pwman.data.tags import TagNew as TagN
 from pwman.util.crypto import CryptoEngine
 from pwman.util.crypto import zerome
 import pwman.util.config as config
@@ -44,6 +45,7 @@ from pwman.ui.tools import CliMenu
 from pwman.ui.tools import CliMenuItem
 from pwman.ui.tools import CLICallback
 from colorama import Fore
+from pwman.ui.base import HelpUI, BaseUI
 
 try:
     import readline
@@ -53,7 +55,7 @@ except ImportError, e:
 
 
 # pylint: disable=R0904
-class PwmanCli(cmd.Cmd):
+class PwmanCliOld(cmd.Cmd, HelpUI, BaseUI):
     """
     UI class for MacOSX
     """
@@ -61,20 +63,11 @@ class PwmanCli(cmd.Cmd):
         if (isinstance(exception, KeyboardInterrupt)):
             print
         else:
-#            traceback.print_exc()
             print "Error: %s " % (exception)
 
-    def do_EOF(self, args):
-        return self.do_exit(args)
-
     def do_exit(self, args):
         """exit the ui"""
-        print
-        # try:
-        #    print "goodbye"
         self._db.close()
-        # except DatabaseException, e:
-        #    self.error(e)
         return True
 
     def get_ids(self, args):
@@ -92,17 +85,17 @@ class PwmanCli(cmd.Cmd):
                     ids += self._db.listnodes()
             else:
                 ids += range(int(m.group(1)),
-                             int(m.group(2))+1)
+                             int(m.group(2)) + 1)
         return ids
 
-    def get_filesystem_path(self, default=""):
-        return tools.getinput("Enter filename: ", default)
+    def get_filesystem_path(self, default="", reader=raw_input):
+        return tools.getinput("Enter filename: ", default, reader=reader)
 
-    def get_username(self, default=""):
-        return tools.getinput("Username: ", default)
+    def get_username(self, default="", reader=raw_input):
+        return tools.getinput("Username: ", default, reader)
 
     def get_password(self, argsgiven, numerics=False, leetify=False,
-                     symbols=False, special_signs=False):
+                     symbols=False, special_signs=False, reader=raw_input):
         """
         in the config file:
         numerics -> numerics
@@ -121,7 +114,7 @@ class PwmanCli(cmd.Cmd):
         # no args given
         password = tools.getpassword("Password (Blank to generate): ",
                                      tools._defaultwidth, False)
-        if len(password) == 0:
+        if not password:
             length = tools.getinput("Password length (default 7): ", "7")
             if length:
                 length = int(length)
@@ -134,15 +127,15 @@ class PwmanCli(cmd.Cmd):
         print "New password: %s" % (password)
         return password
 
-    def get_url(self, default=""):
+    def get_url(self, default="", reader=raw_input):
         return tools.getinput("Url: ", default)
 
-    def get_notes(self, default=""):
+    def get_notes(self, default="", reader=raw_input):
         return tools.getinput("Notes: ", default)
 
     def get_tags(self, default=None):
+        """read node tags from user"""
         defaultstr = ''
-
         if default:
             for t in default:
                 defaultstr += "%s " % (t.get_name())
@@ -175,14 +168,14 @@ class PwmanCli(cmd.Cmd):
     def print_node(self, node):
         width = str(tools._defaultwidth)
         print "Node %d." % (node._id)
-        print ("%"+width+"s %s") % (tools.typeset("Username:", Fore.RED),
-                                    node.get_username())
-        print ("%"+width+"s %s") % (tools.typeset("Password:", Fore.RED),
-                                    node.get_password())
-        print ("%"+width+"s %s") % (tools.typeset("Url:", Fore.RED),
-                                    node.get_url())
-        print ("%"+width+"s %s") % (tools.typeset("Notes:", Fore.RED),
-                                    node.get_notes())
+        print ("%" + width + "s %s") % (tools.typeset("Username:", Fore.RED),
+                                        node.get_username())
+        print ("%" + width + "s %s") % (tools.typeset("Password:", Fore.RED),
+                                        node.get_password())
+        print ("%" + width + "s %s") % (tools.typeset("Url:", Fore.RED),
+                                        node.get_url())
+        print ("%" + width + "s %s") % (tools.typeset("Notes:", Fore.RED),
+                                        node.get_notes())
         print tools.typeset("Tags: ", Fore.RED),
         for t in node.get_tags():
             print " %s \n" % t.get_name(),
@@ -259,9 +252,6 @@ class PwmanCli(cmd.Cmd):
         except Exception, e:
             self.error(e)
 
-    def do_e(self, arg):
-        self.do_edit(arg)
-
     def do_edit(self, arg):
         ids = self.get_ids(arg)
         for i in ids:
@@ -339,12 +329,6 @@ class PwmanCli(cmd.Cmd):
         except Exception, e:
             self.error(e)
 
-    def do_h(self, arg):
-        self.do_help(arg)
-
-    def do_n(self, arg):
-        self.do_new(arg)
-
     def do_new(self, args):
         """
         can override default config settings the following way:
@@ -352,9 +336,9 @@ class PwmanCli(cmd.Cmd):
         pwman> n {'leetify':False, 'numerics':True, 'special_chars':True}
         Password (Blank to generate):
         """
-        errmsg = """could not parse config override, please input some"""\
-                 + """ kind of dictionary, e.g.: n {'leetify':False, """\
-                 + """'numerics':True, 'special_chars':True}"""
+        errmsg = ("could not parse config override, please input some"
+                  " kind of dictionary, e.g.: n {'leetify':False, "
+                  " numerics':True, 'special_chars':True}")
         try:
             username = self.get_username()
             if args:
@@ -388,9 +372,6 @@ class PwmanCli(cmd.Cmd):
         except Exception, e:
             self.error(e)
 
-    def do_p(self, arg):
-        self.do_print(arg)
-
     def do_print(self, arg):
         for i in self.get_ids(arg):
             try:
@@ -399,9 +380,6 @@ class PwmanCli(cmd.Cmd):
             except Exception, e:
                 self.error(e)
 
-    def do_rm(self, arg):
-        self.do_delete(arg)
-
     def do_delete(self, arg):
         ids = self.get_ids(arg)
         try:
@@ -415,14 +393,13 @@ class PwmanCli(cmd.Cmd):
         except Exception, e:
             self.error(e)
 
-    def do_l(self, args):
-        self.do_list(args)
-
-    def do_ls(self, args):
-        self.do_list(args)
-
     def do_list(self, args):
-
+        """
+        TODO: in order to make this code testable
+        The functionality in this method should
+        go to a method that returns a string.
+        This method should only do the printing.
+        """
         if len(args.split()) > 0:
             self.do_clear('')
             self.do_filter(args)
@@ -450,15 +427,15 @@ class PwmanCli(cmd.Cmd):
                 name_len = cols * 2 / 3
                 tagstring_len = cols / 3
                 if len(name) > name_len:
-                    name = name[:name_len-3] + "..."
+                    name = name[:name_len - 3] + "..."
                 if len(tagstring) > tagstring_len:
-                    tagstring = tagstring[:tagstring_len-3] + "..."
+                    tagstring = tagstring[:tagstring_len - 3] + "..."
 
                 fmt = "%%5d. %%-%ds %%-%ds" % (name_len, tagstring_len)
                 print tools.typeset(fmt % (n.get_id(), name, tagstring),
                                     Fore.YELLOW, False)
                 i += 1
-                if i > rows-2:
+                if i > rows - 2:
                     i = 0
                     c = tools.getonechar("Press <Space> for more, "
                                          "or 'Q' to cancel")
@@ -545,9 +522,6 @@ class PwmanCli(cmd.Cmd):
         else:
             print "Can't copy to clipboard, no xsel found in the system!"
 
-    def do_cp(self, args):
-        self.do_copy(args)
-
     def do_open(self, args):
         ids = self.get_ids(args)
         if not args:
@@ -563,144 +537,6 @@ class PwmanCli(cmd.Cmd):
         except Exception, e:
             self.error(e)
 
-    def do_o(self, args):
-        self.do_open(args)
-    ##
-    ## Help functions
-    ##
-
-    def usage(self, string):
-        print "Usage: %s" % (string)
-
-    def help_open(self):
-        self.usage("open <ID>")
-        print "Launch default browser with 'xdg-open url',\n" \
-              + "the url must contain http:// or https://."
-
-    def help_o(self):
-        self.help_open()
-
-    def help_copy(self):
-        self.usage("copy <ID>")
-        print "Copy password to X clipboard (xsel required)"
-
-    def help_cp(self):
-        self.help_copy()
-
-    def help_cls(self):
-        self.usage("cls")
-        print "Clear the Screen from information."
-
-    def help_list(self):
-        self.usage("list <tag> ...")
-        print "List nodes that match current or specified filter." \
-              + " l is an alias."
-
-    def help_EOF(self):
-        self.help_exit()
-
-    def help_delete(self):
-        self.usage("delete <ID|tag> ...")
-        print "Deletes nodes. rm is an alias."
-        self._mult_id_help()
-
-    def help_h(self):
-        self.help_help()
-
-    def help_help(self):
-        self.usage("help [topic]")
-        print "Prints a help message for a command."
-
-    def help_e(self):
-        self.help_edit()
-
-    def help_n(self):
-        self.help_new()
-
-    def help_p(self):
-        self.help_print()
-
-    def help_l(self):
-        self.help_list()
-
-    def help_edit(self):
-        self.usage("edit <ID|tag> ... ")
-        print "Edits a nodes."
-        self._mult_id_help()
-
-    def help_import(self):
-        self.usage("import [filename] ...")
-        print "Imports a nodes from a file."
-
-    def help_export(self):
-        self.usage("export <ID|tag> ... ")
-        print "Exports a list of ids to an external format. If no IDs or" \
-              + " tags are specified, then all nodes under the current" \
-              + " filter are exported."
-        self._mult_id_help()
-
-    def help_new(self):
-        self.usage("new")
-        print """Creates a new node.,
-You can override default config settings the following way:
-pwman> n {'leetify':False, 'numerics':True}"""
-
-    def help_rm(self):
-        self.help_delete()
-
-    def help_print(self):
-        self.usage("print <ID|tag> ...")
-        print "Displays a node. ",
-        self._mult_id_help()
-
-    def _mult_id_help(self):
-        print "Multiple ids and nodes can be specified, separated by a space."\
-              + "A range of ids can be specified in the format n-N. e.g. " \
-              + "'10-20' would specify all nodes having ids from 10 to 20 " \
-              + "inclusive. Tags are considered one-by-one. e.g. 'foo 2 bar'" \
-              + " would yield to all nodes with tag 'foo', node 2 and all "\
-              + "nodes with tag 'bar'."
-
-    def help_exit(self):
-        self.usage("exit")
-        print "Exits the application."
-
-    def help_save(self):
-        self.usage("save [filename]")
-        print "Saves the current configuration to [filename]. If no filename "\
-              + "is given, the configuration is saved to the file from which "\
-              + "the initial configuration was loaded."
-
-    def help_set(self):
-        self.usage("set [configoption] [value]")
-        print "Sets a configuration option. If no value is specified, the "\
-              + "current value for [configoption] is output. If neither "\
-              + "[configoption] nor [value] are specified, the whole current "\
-              + "configuration is output. [configoption] must be of the "\
-              + "format <section>.<option>"
-
-    def help_passwd(self):
-        self.usage("passwd")
-        print "Changes the password on the database. "
-
-    def help_forget(self):
-        self.usage("forget")
-        print "Forgets the database password. Your password will need to " \
-              + "be reentered before accessing the database again."
-
-    def help_clear(self):
-        self.usage("clear")
-        print "Clears the filter criteria. "
-
-    def help_filter(self):
-        self.usage("filter <tag> ...")
-        print "Filters nodes on tag. Arguments can be zero or more tags. " \
-              + "Displays current tags if called without arguments."
-
-    def help_tags(self):
-        self.usage("tags")
-        print "Displays all tags in used in the database."
-
     def postloop(self):
         try:
             readline.write_history_file(self._historyfile)
@@ -715,7 +551,8 @@ pwman> n {'leetify':False, 'numerics':True}"""
         _dbwarning = "\n*** WARNNING: You are using the old database format" \
             + " which is unsecure." \
             + " It's highly recommended to switch to the new database " \
-            + "format." \
+            + "format. Do note: support for this DB format will be dropped in"\
+            + " v0.5." \
             + " Check the help (pwman3 -h) or look at the manpage which" \
             + " explains how to proceed. ***"
 
@@ -739,40 +576,15 @@ pwman> n {'leetify':False, 'numerics':True}"""
         self.prompt = "!pwman> "
 
 
-class PwmanCliNew(PwmanCli):
+class BaseCommands(PwmanCliOld):
     """
-    inherit from the old class, override
+    Inherit from the old class, override
     all the methods related to tags, and
     newer Node format, so backward compatability is kept...
+    Commands defined here, can have aliases definded in Aliases.
+    You can define the aliases here too, but it makes
+    the class code really long and unclear.
     """
-
-    def __init__(self, db, hasxsel):
-        """
-        initialize CLI interface, set up the DB
-        connecion, see if we have xsel ...
-        """
-        cmd.Cmd.__init__(self)
-        self.intro = "%s %s (c) visit: %s" % (pwman.appname, pwman.version,
-                                              pwman.website)
-        self._historyfile = config.get_value("Readline", "history")
-        self.hasxsel = hasxsel
-        try:
-            enc = CryptoEngine.get()
-            #enc.set_callback(CLICallback())
-            enc._callback = CLICallback()
-            self._db = db
-            self._db.open()
-        except Exception, e:
-            self.error(e)
-            sys.exit(1)
-
-        try:
-            readline.read_history_file(self._historyfile)
-        except IOError, e:
-            pass
-
-        self.prompt = "pwman> "
-
     def do_copy(self, args):
         if self.hasxsel:
             ids = self.get_ids(args)
@@ -843,14 +655,14 @@ class PwmanCliNew(PwmanCli):
     def print_node(self, node):
         width = str(tools._defaultwidth)
         print "Node %d." % (node._id)
-        print ("%"+width+"s %s") % (tools.typeset("Username:", Fore.RED),
-                                    node.username)
-        print ("%"+width+"s %s") % (tools.typeset("Password:", Fore.RED),
-                                    node.password)
-        print ("%"+width+"s %s") % (tools.typeset("Url:", Fore.RED),
-                                    node.url)
-        print ("%"+width+"s %s") % (tools.typeset("Notes:", Fore.RED),
-                                    node.notes)
+        print ("%" + width + "s %s") % (tools.typeset("Username:", Fore.RED),
+                                        node.username)
+        print ("%" + width + "s %s") % (tools.typeset("Password:", Fore.RED),
+                                        node.password)
+        print ("%" + width + "s %s") % (tools.typeset("Url:", Fore.RED),
+                                        node.url)
+        print ("%" + width + "s %s") % (tools.typeset("Notes:", Fore.RED),
+                                        node.notes)
         print tools.typeset("Tags: ", Fore.RED),
         for t in node.tags:
             print " %s " % t
@@ -880,33 +692,32 @@ class PwmanCliNew(PwmanCli):
             waituntil_enter(heardEnter, flushtimeout)
 
     def do_tags(self, arg):
-        tags = self._db.listtags()
-        # request password
         enc = CryptoEngine.get()
         if not enc.alive():
             enc._getcipher()
-        print "Tags: ",
-        if len(tags) == 0:
-            print "None",
-        for t in tags:
-            print "%s " % t,
-        print
+        print "Tags: \n",
+        t = self._tags(enc)
+        print '\n'.join(t)
 
     def get_tags(self, default=None):
+        """read tags from user"""
         defaultstr = ''
         if default:
             for t in default:
                 defaultstr += "%s " % (t)
         else:
-            tags = self._db.currenttags()
+            # tags = self._db.currenttags()
+            tags = self._db._filtertags
             for t in tags:
                 defaultstr += "%s " % (t)
 
-        strings = []
+        # strings = []
         tags = self._db.listtags(True)
-        for t in tags:
+        #for t in tags:
             # strings.append(t.get_name())
-            strings.append(t)
+        #    strings.append(t)
+
+        strings = [t for t in tags]
 
         def complete(text, state):
             count = 0
@@ -917,9 +728,9 @@ class PwmanCliNew(PwmanCli):
                     else:
                         count += 1
 
-        taglist = tools.getinput("Tags: ", defaultstr, complete)
+        taglist = tools.getinput("Tags: ", defaultstr, completer=complete)
         tagstrings = taglist.split()
-        tags = [Tag(tn) for tn in tagstrings]
+        tags = [TagN(tn) for tn in tagstrings]
 
         return tags
 
@@ -954,16 +765,16 @@ class PwmanCliNew(PwmanCli):
                 name_len = cols * 2 / 3
                 tagstring_len = cols / 3
                 if len(name) > name_len:
-                    name = name[:name_len-3] + "..."
+                    name = name[:name_len - 3] + "..."
                 if len(tagstring) > tagstring_len:
-                    tagstring = tagstring[:tagstring_len-3] + "..."
+                    tagstring = tagstring[:tagstring_len - 3] + "..."
                 fmt = "%%5d. %%-%ds %%-%ds" % (name_len, tagstring_len)
                 formatted_entry = tools.typeset(fmt % (n._id,
                                                 name, tagstring),
                                                 Fore.YELLOW, False)
                 print formatted_entry
                 i += 1
-                if i > rows-2:
+                if i > rows - 2:
                     i = 0
                     c = tools.getonechar("Press <Space> for more,"
                                          " or 'Q' to cancel")
@@ -978,7 +789,7 @@ class PwmanCliNew(PwmanCli):
         try:
             tags = []
             for ts in tagstrings:
-                tags.append(Tag(ts))
+                tags.append(TagN(ts))
             self._db.filter(tags)
             tags = self._db.currenttags()
             print "Current tags: ",
@@ -997,9 +808,9 @@ class PwmanCliNew(PwmanCli):
         pwman> n {'leetify':False, 'numerics':True, 'special_chars':True}
         Password (Blank to generate):
         """
-        errmsg = """could not parse config override, please input some"""\
-                 + """ kind of dictionary, e.g.: n {'leetify':False, """\
-                 + """'numerics':True, 'special_chars':True}"""
+        errmsg = ("could not parse config override, please input some"
+                  " kind of dictionary, e.g.: n {'leetify':False, "
+                  " numerics':True, 'special_chars':True}")
         try:
             username = self.get_username()
             if args:
@@ -1048,10 +859,81 @@ class PwmanCliNew(PwmanCli):
         try:
             nodes = self._db.getnodes(ids)
             for n in nodes:
-                b = tools.getyesno("Are you sure you want to delete '%s@%s'?"
-                                   % (n.username, n.url), False)
+                try:
+                    b = tools.getyesno(("Are you sure you want to"
+                                        " delete '%s@%s'?"
+                                        ) % (n.username, n.url), False)
+                except NameError:
+                    pass
                 if b is True:
                     self._db.removenodes([n])
                     print "%s@%s deleted" % (n.username, n.url)
         except Exception, e:
             self.error(e)
+
+
+class Aliases(BaseCommands, PwmanCliOld):
+    """
+    Define all the alias you want here...
+    """
+    def do_cp(self, args):
+        self.do_copy(args)
+
+    def do_e(self, arg):
+        self.do_edit(arg)
+
+    def do_EOF(self, args):
+        return self.do_exit(args)
+
+    def do_l(self, args):
+        self.do_list(args)
+
+    def do_ls(self, args):
+        self.do_list(args)
+
+    def do_p(self, arg):
+        self.do_print(arg)
+
+    def do_rm(self, arg):
+        self.do_delete(arg)
+
+    def do_o(self, args):
+        self.do_open(args)
+
+    def do_h(self, arg):
+        self.do_help(arg)
+
+    def do_n(self, arg):
+        self.do_new(arg)
+
+
+class PwmanCliNew(Aliases, BaseCommands):
+    """
+    Inherit from the BaseCommands and Aliases
+    """
+    def __init__(self, db, hasxsel):
+        """
+        initialize CLI interface, set up the DB
+        connecion, see if we have xsel ...
+        """
+        cmd.Cmd.__init__(self)
+        self.intro = "%s %s (c) visit: %s" % (pwman.appname, pwman.version,
+                                              pwman.website)
+        self._historyfile = config.get_value("Readline", "history")
+        self.hasxsel = hasxsel
+        try:
+            enc = CryptoEngine.get()
+            #enc.set_callback(CLICallback())
+            enc._callback = CLICallback()
+            self._db = db
+            self._db.open()
+        except Exception, e:
+            self.error(e)
+            sys.exit(1)
+
+        try:
+            readline.read_history_file(self._historyfile)
+        except IOError, e:
+            pass
+
+        self.prompt = "pwman> "
