@@ -28,7 +28,7 @@ import sys
 import struct
 import os
 import colorama
-from pwman.data.tags import Tag
+from pwman.data.tags import TagNew as Tag
 
 if sys.platform != 'win32':
     import termios
@@ -78,7 +78,7 @@ def typeset(text, color, bold=False, underline=False):
         text = colorama.Style.BRIGHT + text
     if underline and not 'win32' in sys.platform:
         text = ANSI.Underscore + text
-    return color+text+colorama.Style.RESET_ALL
+    return color + text + colorama.Style.RESET_ALL
 
 
 def select(question, possible):
@@ -86,11 +86,11 @@ def select(question, possible):
     select input from user
     """
     for i in range(0, len(possible)):
-        print ("%d - %-"+str(_defaultwidth)+"s") % (i+1, possible[i])
+        print ("%d - %-" + str(_defaultwidth) + "s") % (i + 1, possible[i])
     while 1:
         uinput = getonechar(question)
-        if uinput.isdigit() and int(uinput) in range(1, len(possible)+1):
-            return possible[int(uinput)-1]
+        if uinput.isdigit() and int(uinput) in range(1, len(possible) + 1):
+            return possible[int(uinput) - 1]
 
 
 def text_to_clipboards(text):
@@ -138,16 +138,17 @@ def open_url(link, macosx=False):
         print "Executing open_url failed with:\n", e
 
 
-def getpassword(question, width=_defaultwidth, echo=False):
+def getpassword(question, width=_defaultwidth, echo=False,
+                reader=getpass.getpass):
     if echo:
         print question.ljust(width),
         return sys.stdin.readline().rstrip()
     else:
-        while 1:
-            a1 = getpass.getpass(question.ljust(width))
-            if len(a1) == 0:
+        while True:
+            a1 = reader(question.ljust(width))
+            if not a1:
                 return a1
-            a2 = getpass.getpass("[Repeat] %s" % (question.ljust(width)))
+            a2 = reader("[Repeat] %s" % (question.ljust(width)))
             if a1 == a2:
                 return a1
             else:
@@ -162,22 +163,30 @@ def gettermsize():
     return rows, cols
 
 
-def getinput(question, default="", completer=None, width=_defaultwidth):
-    if not _readline_available:
-        return raw_input(question.ljust(width))
-    else:
-        def defaulter():
-            """define default behavior startup"""
-            if _readline_available:
-                readline.insert_text(default)
-            readline.set_startup_hook(defaulter)
-            oldcompleter = readline.get_completer()
-            readline.set_completer(completer)
+def getinput(question, default="", reader=raw_input,
+             completer=None, width=_defaultwidth):
+    """
+    http://stackoverflow.com/questions/2617057/\
+            supply-inputs-to-python-unittests
+    """
+    if reader == raw_input:
+        if not _readline_available:
+            return raw_input(question.ljust(width))
+        else:
+            def defaulter():
+                """define default behavior startup"""
+                if _readline_available:
+                    readline.insert_text(default)
+                readline.set_startup_hook(defaulter)
+                oldcompleter = readline.get_completer()
+                readline.set_completer(completer)
 
-        x = raw_input(question.ljust(width))
-        readline.set_completer(completer)
-        readline.set_startup_hook()
-        return x
+            x = raw_input(question.ljust(width))
+            readline.set_completer(completer)
+            readline.set_startup_hook()
+            return x
+    else:
+        return reader()
 
 
 def getyesno(question, defaultyes=False, width=_defaultwidth):
@@ -186,6 +195,7 @@ def getyesno(question, defaultyes=False, width=_defaultwidth):
     else:
         default = "[y/N]"
     ch = getonechar("%s %s" % (question, default), width)
+
     if (ch == '\n'):
         if (defaultyes):
             return True
@@ -227,8 +237,8 @@ class CliMenu(object):
                 else:
                     currentstr = current
 
-                print ("%d - %-"+str(_defaultwidth)
-                       + "s %s") % (i, x.name+":",
+                print ("%d - %-" + str(_defaultwidth)
+                       + "s %s") % (i, x.name + ":",
                                     currentstr)
             print "%c - Finish editing" % ('X')
             option = getonechar("Enter your choice:")
@@ -272,15 +282,15 @@ class CliMenu(object):
                 if type(current) == list:
                     for c in current:
                         try:
-                            currentstr += ' '+c
+                            currentstr += ' ' + c
                         except TypeError:
-                            currentstr += ' '+c._name
+                            currentstr += ' ' + c._name
 
                 else:
                     currentstr = current
 
-                print ("%d - %-"+str(_defaultwidth)
-                       + "s %s") % (i, x.name+":",
+                print ("%d - %-" + str(_defaultwidth)
+                       + "s %s") % (i, x.name + ":",
                                     currentstr)
             print "%c - Finish editing" % ('X')
             option = getonechar("Enter your choice:")
@@ -294,14 +304,17 @@ class CliMenu(object):
                 # is created.
                 if selection == 0:
                     new_node.username = getinput("Username:")
-                    self.items[2].getter = new_node.username
+                    self.items[0].getter = new_node.username
+                    self.items[0].setter = new_node.username
                 elif selection == 1:  # for password
                     value = self.items[selection].editor(0)
                     new_node.password = value
-                    self.items[2].getter = new_node.password
+                    self.items[1].getter = new_node.password
+                    self.items[1].setter = new_node.password
                 elif selection == 2:
-                    new_node.notes = getinput("Url:")
+                    new_node.url = getinput("Url:")
                     self.items[2].getter = new_node.url
+                    self.items[2].setter = new_node.url
                 elif selection == 3:  # for notes
                     new_node.notes = getinput("Notes:")
                     self.items[3].getter = new_node.notes
@@ -326,12 +339,19 @@ def getonechar(question, width=_defaultwidth):
     sys.stdout.flush()
 
     fd = sys.stdin.fileno()
-    tty_mode = tty.tcgetattr(fd)
-    tty.setcbreak(fd)
+    # tty module exists only if we are on Posix
+    try:
+        tty_mode = tty.tcgetattr(fd)
+        tty.setcbreak(fd)
+    except NameError:
+        pass
     try:
         ch = os.read(fd, 1)
     finally:
-        tty.tcsetattr(fd, tty.TCSAFLUSH, tty_mode)
+        try:
+            tty.tcsetattr(fd, tty.TCSAFLUSH, tty_mode)
+        except NameError:
+            pass
     print ch
     return ch
 
